function [W,qspace,pspace] = wignerX(rhoxx,finalgrid);

% This calculates the Wigner function for a density matrix in the x basis
% rho is the density matrix. Nexp2 is the power of 2 to be used in the FFT. 
% (ie. the number of sample points, which is a power of two) If
% more sample points are required to obtain the momentum space range given by 
% xymax (the momentum and position ranges are set equal) then a message is 
% produced.

Nx=length(rhoxx);
% The max p and max y are now set to be the same.
Y=sqrt(Nx*2*pi);  
plim=Y/4; qlim=Y/2; qn=Nx; pn=Nx; 

Wfull=zeros(pn,qn); 

%second calculate the Wigner function by fourier transforming

fxin=[1:Nx/2 Nx/2:-1:1];
gxin=[(Nx-1):-2:1 -1:-2:(-(Nx-1))];
for xin=1:Nx,
  val=fix((Nx-(2*fxin(xin)-1))/2);
  yvec=[zeros(val+1,1); diag(fliplr(rhoxx),gxin(xin)); zeros(val,1)];
  Wfull(:,xin) = 2*pi*(Y/(2*Nx))*fftshift(fft(fftshift(yvec)));
end;

% use interpolation to return the desired ranges

lqspace=linspace(-qlim,qlim,qn); 
lpspace=linspace(-plim,plim,pn); 

xymax = plim;
[Lqspace Lpspace]=meshgrid(lqspace,lpspace);
[qspace pspace]=meshgrid(linspace(-xymax,xymax,finalgrid),linspace(-xymax,xymax,finalgrid));
W=interp2(Lqspace,Lpspace,Wfull,qspace,pspace);
